#--
################################################################################
#                      Copyright (C) 2011 Travis Herrick                       #
################################################################################
#                                                                              #
#                                 \v^V,^!v\^/                                  #
#                                 ~%       %~                                  #
#                                 {  _   _  }                                  #
#                                 (  *   -  )                                  #
#                                 |    /    |                                  #
#                                  \   _,  /                                   #
#                                   \__.__/                                    #
#                                                                              #
################################################################################
# This program is free software: you can redistribute it                       #
# and/or modify it under the terms of the GNU General Public License           #
# as published by the Free Software Foundation,                                #
# either version 3 of the License, or (at your option) any later version.      #
#                                                                              #
# Commercial licensing may be available for a fee under a different license.   #
################################################################################
# This program is distributed in the hope that it will be useful,              #
# but WITHOUT ANY WARRANTY;                                                    #
# without even the implied warranty of MERCHANTABILITY                         #
# or FITNESS FOR A PARTICULAR PURPOSE.                                         #
# See the GNU General Public License for more details.                         #
#                                                                              #
# You should have received a copy of the GNU General Public License            #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.        #
################################################################################
#++

require_relative 'require'

class ModuleFunctionsTest < Test::Unit::TestCase
  ITEMS = {
    :public_methods => [
      :my_public_module_method,
      :my_default_module_function,
      :my_public_module_function,
      :my_protected_module_function,
      :my_private_module_function,
    ],
    :non_public_methods => [
      :my_protected_module_method,
      :my_private_module_method,
    ],
  }

  def setup
    super
    @class = MyClass
    @module = ModuleFunctions
    @my_module = MyModule
    @empty = EmptyModule
  end

  def test_import_public_methods
    assert_nothing_raised {
      @module.import_public_methods(@empty, @my_module) }
    assert_responses @empty, ITEMS[:public_methods]
    assert_no_responses @empty, ITEMS[:non_public_methods]
  end

  def test_import_into_class_should_fail
    assert_raise(ArgumentError) {
      @module.import_public_methods(@class, @my_module) }
  end

  def test_import_from_class_should_fail
    assert_raise(ArgumentError) {
      @module.import_public_methods(@my_module, @class) }
  end


  ############################################################################
  private
  ############################################################################

  def assert_responses(object, methods)
    methods.each do |method|
      assert_respond_to object, method
    end
  end

  def assert_no_responses(object, methods)
    methods.each do |method|
      assert !object.respond_to?(method),
        "#{object.class} was not expected to respond to #{method}."
    end
  end
end
